@@ -7,7 +7,7 @@ from .models import (
     Order,
     RoutePlan,
     SchedulingInfo,
-    UnavailableTime,
 )
 
-from .api import OptimoAPI, OptimoError
+from .api import OptimoAPI
+from .errors import OptimoError
